/** @module @codefresh-io/cf-variable-resolver */
/**
 * Logger
 * @typedef {Object} Logger
 * @property {function(string):undefined} info - log info level
 * @property {function(string):undefined} warn - log warn level
 */
/**
 * getContextFn
 * @typedef {function(string, Object, Logger): Promise<*>} getContextFn
 */
/**
 * Resolver
 * @typedef {Object} Resolver
 * @property {function(string, getContextFn, Logger):Promise<string>} resolve - resolution function, gets: variable, getContextFn, and logger
 */
/**
 * Resolution
 * @typedef {Object} Resolution
 * @property {number} index - index
 * @property {string} key - key
 * @property {boolean} resolved - resolved
 * @property {string} value - value
 */
/**
 * PathResolution
 * @typedef {Object} PathResolution
 * @property {string} path - path
 * @property {Resolution} resolution - resolution
 */
/**
 * Variable resolver options
 * @typedef {Object} Options
 * @property {number | undefined} singleResolutionTimeout - sets the timeout for single variable resolution
 * @property {number | undefined} completeResolutionFlowTimeout - sets the timeout for the entire resolution
 * @property {Logger | undefined} log - sets the timeout for the entire resolution
 * @property {undefined | function(string, Object, Logger): Promise<*>} getContext - a function used by the variable resolver to retrieve contexts
 * @property {undefined | function(Resolution, Logger): Promise<void>} addNewMask - a function used by the variable resolver to add a new mask
 */
/**
 * resolution options
 * @typedef {Object} ResolutionOptions
 * @property {number | undefined} timeout - sets the timeout for single variable resolution
 * @property {Logger | undefined} log - sets the timeout for the entire resolution
 * @property {function(string, Object, Logger): Promise<*>} getContext - a function used by the variable resolver to retrieve contexts
 * @property {function(Resolution, Logger): Promise<void>} addNewMask - a function used by the variable resolver to add a new mask
 */
/**
 * VariableResolver
 * @typedef {Object} VariableResolver
 * @property {function(string, ResolutionOptions): Promise<Resolution[]>} resolve - resolve a string variable
 * @property {function(Object, ResolutionOptions): Promise<PathResolution[]>} recursive - recursively resolve secrets on an object
 * @property {function(getContextFn)} setGetContext - sets the getContext function of the variable resolver
 * @property {function(function(Resolution, Logger): Promise<void>)} setAddNewMask - sets the addNewMask function of the variable resolver
 */
/**
 * Builds a new variable resolver
 * @param {Object.<string, Resolver>} resolvers resolvers
 * @param {Options} options options
 * @returns {VariableResolver} variableResolver
 */
export function build(resolvers: {
    [x: string]: Resolver;
}, options: Options): VariableResolver;
/**
 * Logger
 */
export type Logger = {
    /**
     * - log info level
     */
    info: (arg0: string) => undefined;
    /**
     * - log warn level
     */
    warn: (arg0: string) => undefined;
};
/**
 * getContextFn
 */
export type getContextFn = (arg0: string, arg1: any, arg2: Logger) => Promise<any>;
/**
 * Resolver
 */
export type Resolver = {
    /**
     * - resolution function, gets: variable, getContextFn, and logger
     */
    resolve: (arg0: string, arg1: getContextFn, arg2: Logger) => Promise<string>;
};
/**
 * Resolution
 */
export type Resolution = {
    /**
     * - index
     */
    index: number;
    /**
     * - key
     */
    key: string;
    /**
     * - resolved
     */
    resolved: boolean;
    /**
     * - value
     */
    value: string;
};
/**
 * PathResolution
 */
export type PathResolution = {
    /**
     * - path
     */
    path: string;
    /**
     * - resolution
     */
    resolution: Resolution;
};
/**
 * Variable resolver options
 */
export type Options = {
    /**
     * - sets the timeout for single variable resolution
     */
    singleResolutionTimeout: number | undefined;
    /**
     * - sets the timeout for the entire resolution
     */
    completeResolutionFlowTimeout: number | undefined;
    /**
     * - sets the timeout for the entire resolution
     */
    log: Logger | undefined;
    /**
     * - a function used by the variable resolver to retrieve contexts
     */
    getContext: undefined | ((arg0: string, arg1: any, arg2: Logger) => Promise<any>);
    /**
     * - a function used by the variable resolver to add a new mask
     */
    addNewMask: undefined | ((arg0: Resolution, arg1: Logger) => Promise<void>);
};
/**
 * resolution options
 */
export type ResolutionOptions = {
    /**
     * - sets the timeout for single variable resolution
     */
    timeout: number | undefined;
    /**
     * - sets the timeout for the entire resolution
     */
    log: Logger | undefined;
    /**
     * - a function used by the variable resolver to retrieve contexts
     */
    getContext: (arg0: string, arg1: any, arg2: Logger) => Promise<any>;
    /**
     * - a function used by the variable resolver to add a new mask
     */
    addNewMask: (arg0: Resolution, arg1: Logger) => Promise<void>;
};
/**
 * VariableResolver
 */
export type VariableResolver = {
    /**
     * - resolve a string variable
     */
    resolve: (arg0: string, arg1: ResolutionOptions) => Promise<Resolution[]>;
    /**
     * - recursively resolve secrets on an object
     */
    recursive: (arg0: any, arg1: ResolutionOptions) => Promise<PathResolution[]>;
    /**
     * - sets the getContext function of the variable resolver
     */
    setGetContext: (arg0: getContextFn) => any;
    /**
     * - sets the addNewMask function of the variable resolver
     */
    setAddNewMask: (arg0: (arg0: Resolution, arg1: Logger) => Promise<void>) => any;
};
/**
 * resolveKey options
 */
export type ResolveKeyOptions = {
    /**
     * - resolvers
     */
    resolvers: {
        [x: string]: Resolver;
    };
    /**
     * - key
     */
    key: string;
    /**
     * - index
     */
    index: number;
    /**
     * - timeout for single variable resolution
     */
    timeout: number | null;
    /**
     * - logger
     */
    log: Logger;
    /**
     * - a function used by the variable resolver to retrieve contexts
     */
    getContext: (arg0: string, arg1: any, arg2: Logger) => Promise<any>;
    /**
     * - a function used by the variable resolver to add a new mask
     */
    addNewMask: (arg0: Resolution, arg1: Logger) => Promise<void> | undefined;
};
declare const Promise: any;
export {};
